<?php

namespace App\Factory;

use App\Exception;

class ConsumerFactory {
    
    private $connection;
    
    public function __construct (\Doctrine\DBAL\Connection $connection) {
        $this->connection = $connection;
    }
    
    public function getConsumerByEmail($email, $standardClient) {
        $sql = "
            SELECT CDCLIENTE, CDCONSUMIDOR,
			       NMCONSUMIDOR, NRCPFRESPCON,
			       CDTIPOCONS, NRRGCONSUMID,
			       DSEMAILCONS, DTNASCCONS,
			       NRCEPCONS, DSEXIBENDE, 
			       'PRINCIPAL' AS CDENDECONS,
	 		       CDPAIS, DSENDECONS,
	 		       NRENDECONS, DSCOMPLENDECONS,
	 		       NRTELECONS, NRCELULARCONS,
	 		       DSREFERENDECONS,
	 		       CDCONSFACEBOOK, DSENDEIMGCONS,
	 		       CDTOKENCONSFB, DTEXPTOKENFB,
	 		       NRLATCONS, NRLONGCONS
		      FROM CONSUMIDOR
		     WHERE LOWER(DSEMAILCONS) = LOWER(:DSEMAILCONS)
		       AND CDCLIENTE          = :CDCLIENTE";
		    
        $params = [
            ":DSEMAILCONS" => strtolower($email),
	        ":CDCLIENTE"   => $standardClient
        ];
        
        return $this->connection->fetchAssoc($sql, $params);
    }
    
    public function getConsumerByPhone($phone, $standardClient) {
        $sql = "
            SELECT CDCLIENTE, CDCONSUMIDOR,
		           NMCONSUMIDOR, NRCPFRESPCON,
		           CDTIPOCONS, NRRGCONSUMID,
		           DSEMAILCONS, DTNASCCONS,
		           NRCEPCONS, DSEXIBENDE,
		           'PRINCIPAL' AS CDENDECONS, 
		           CDPAIS,
		           DSENDECONS, NRENDECONS,
		           DSCOMPLENDECONS, NRTELECONS,
		           NRCELULARCONS, DSREFERENDECONS,
	 		       CDCONSFACEBOOK, DSENDEIMGCONS,
	 		       CDTOKENCONSFB, DTEXPTOKENFB,
	 		       NRLATCONS, NRLONGCONS

              FROM CONSUMIDOR

            WHERE (NRTELECONS    = :NRTELECONS
                OR NRTELE2CONS   = :NRCELULARCONS
                OR NRCELULARCONS = :NRCELULARCONS)
		      AND CDCLIENTE      = :CDCLIENTE";
        
        $params = [
            ":NRTELECONS"    => $phone,
            ":NRCELULARCONS" => $phone,
	        ":CDCLIENTE"     => $standardClient
        ];
        
        return $this->connection->fetchAssoc($sql, $params);
    }
    
    public function getCityCode($state, $city) {
        $sql = "
            SELECT CDMUNICIPIO 
    	      FROM MUNICIPIO
    	     WHERE UPPER(SGESTADO) = :SGESTADO
               AND CDPAIS   = :CDPAIS
    		   AND NVL((SELECT CDMUNICIPIO
    		            FROM MUNICIPIO
    		           WHERE UPPER(NMMUNICIPIO) = :NMMUNICIPIO
                         AND UPPER(SGESTADO) = :SGESTADO
                         AND CDPAIS   = :CDPAIS
    		             AND ROWNUM = 1),
    		         (SELECT CDMUNICIPIO
    		            FROM MUNICIPIO
    		           WHERE UPPER(NMMUNICIPIO) LIKE '%'||:NMMUNICIPIO||'%'
                         AND UPPER(SGESTADO) = :SGESTADO
                         AND CDPAIS   = :CDPAIS
    		             AND ROWNUM = 1)) = CDMUNICIPIO";
    	
    	$params = [
    	    ":CDPAIS"      => "0055",
    	    ":SGESTADO"    => strtoupper($state),
    	    ":NMMUNICIPIO" => strtoupper($this->removeAcentos($city))
    	];
    	
    	$result = $this->connection->fetchAll($sql, $params);
    	
    	if(empty($result)) 
    	    return null;
    	
    	return $result[0]["CDMUNICIPIO"];
    }
    
    public function getNeighborhoodCode ($address) {
        
        $sql = "
            SELECT CDBAIRRO
              FROM BAIRRO
             WHERE CDPAIS          = :CDPAIS
               AND SGESTADO        = :SGESTADO
               AND CDMUNICIPIO     = :CDMUNICIPIO
               AND UPPER(NMBAIRRO) = UPPER(:NMBAIRRO)";
        
        $params = [
            ":CDPAIS"      => "0055",
            ":SGESTADO"    => $address["state"],
            ":CDMUNICIPIO" => $address["city"],
            ":NMBAIRRO"    => $address["neighborhood"]
        ];
        
        $result = $this->connection->fetchAll($sql, $params);
        
        if (empty($result))
            return null;
        
        return $result[0]["CDBAIRRO"];
    }
    
    public function registerConsumer($consumerDetails) {
        $sql = "
            INSERT INTO CONSUMIDOR (
                CDCLIENTE, CDCONSUMIDOR, NMCONSUMIDOR, IDSITCONSUMI,
                CDCCUSCLIE, IDCONSUMIDOR, IDATUCONSUMI, IDTPVENDACONS,
                IDIMPCPFCUPOM, IDCADCONFLIBCON, IDPERCONSPRODEX, IDTPSELMANHA,
                IDTPSEALMOCO, IDTPSELTARDE, IDCRACHAMESTRE, CDPAIS,
                SGESTADO, CDMUNICIPIO, CDTIPOCONS, CDIDCONSUMID,
                DSENDECONS, NMBAIRCONS, NRCEPCONS, NRTELECONS,
                DSEMAILCONS, NMRESPCONS, DSREFERENDECONS, DSINFOCONS
            )
            VALUES (
                :CDCLIENTE, :CDCONSUMIDOR, :NMCONSUMIDOR, :IDSITCONSUMI,
                :CDCCUSCLIE, :IDCONSUMIDOR, :IDATUCONSUMI, :IDTPVENDACONS,
                :IDIMPCPFCUPOM, :IDCADCONFLIBCON, :IDPERCONSPRODEX, :IDTPSELMANHA,
                :IDTPSEALMOCO, :IDTPSELTARDE, :IDCRACHAMESTRE, :CDPAIS,
                :SGESTADO, :CDMUNICIPIO, :CDTIPOCONS, :CDIDCONSUMID,
                :DSENDECONS, :NMBAIRCONS, :NRCEPCONS, :NRTELECONS,
                :DSEMAILCONS, :NMRESPCONS, :DSREFERENDECONS, :DSINFOCONS
            )";
        
        $params = [
            ":CDCLIENTE"       => $consumerDetails["CDCLIENTE"],
            ":CDCONSUMIDOR"    => $consumerDetails["CDCONSUMIDOR"],
            ":NMCONSUMIDOR"    => substr($consumerDetails["name"], 0, 50),
            ":IDSITCONSUMI"    => "1",
            ":CDCCUSCLIE"      => $consumerDetails["CDCCUSCLIE"],
            ":IDCONSUMIDOR"    => "C",
            ":IDATUCONSUMI"    => "S",
            ":IDTPVENDACONS"   => "7", 
            ":IDIMPCPFCUPOM"   => "S", 
            ":IDCADCONFLIBCON" => "N", 
            ":IDPERCONSPRODEX" => "S", 
            ":IDTPSELMANHA"    => "N", 
            ":IDTPSEALMOCO"    => "N", 
            ":IDTPSELTARDE"    => "N", 
            ":IDCRACHAMESTRE"  => "N", 
            ":CDPAIS"          => "0055",
            ":SGESTADO"        => $consumerDetails["state"],
            ":CDMUNICIPIO"     => $consumerDetails["CDMUNICIPIO"],
            ":CDTIPOCONS"      => $consumerDetails["CDTIPOCONS"],
            ":CDIDCONSUMID"    => $consumerDetails["CDCONSUMIDOR"],
            ":DSENDECONS"      => substr($consumerDetails["street"], 0, 50),
            ":NMBAIRCONS"      => "",
            ":NRCEPCONS"       => preg_replace("/[^0-9]/", "", $consumerDetails["zipCode"]),
            ":NRTELECONS"      => preg_replace("/[^0-9]/", "", $consumerDetails["telephone"]),
            ":DSEMAILCONS"     => substr($consumerDetails["email"], 0, 50),
            ":NMRESPCONS"      => substr($consumerDetails["name"], 0, 50),
            ":DSREFERENDECONS" => $consumerDetails["complement"],
            ":DSINFOCONS"      => "SDE - Incluido"
        ];
        
        $this->connection->executeQuery($sql, $params);
        
        $sql = "
            SELECT *
              FROM CONSUMIDOR
             WHERE CDCLIENTE    = :CDCLIENTE
               AND CDCONSUMIDOR = :CDCONSUMIDOR";
            
        $params = [
            ":CDCLIENTE"    => $consumerDetails["CDCLIENTE"],
            ":CDCONSUMIDOR" => $consumerDetails["CDCONSUMIDOR"]
        ];
        
        return $this->connection->fetchAssoc($sql, $params);
    }
    
    public function removeAcentos ($texto) {
        return preg_replace(array("/(á|à|ã|â|ä)/","/(Á|À|Ã|Â|Ä)/","/(é|è|ê|ë)/","/(É|È|Ê|Ë)/","/(í|ì|î|ï)/","/(Í|Ì|Î|Ï)/","/(ó|ò|õ|ô|ö)/","/(Ó|Ò|Õ|Ô|Ö)/","/(ú|ù|û|ü)/","/(Ú|Ù|Û|Ü)/","/(ñ)/","/(Ñ)/","/(ç)/","/(Ç)/"),explode(" ","a A e E i I o O u U n N c C"),$texto);
    }
    
}
